@@ -4,6 +4,7 @@ use std::{
     fmt::{Display, Formatter, Result as FmtResult},
     iter::{once, repeat},
     num::{IntErrorKind, NonZeroUsize},
+    ops::Not,
     str::FromStr,
 };
 
@@ -38,6 +39,16 @@ impl Display for Color {
     }
 }
 
+impl Not for Color {
+    type Output = Self;
+    fn not(self) -> Self::Output {
+        match self {
+            Self::White => Self::Black,
+            Self::Black => Self::White,
+        }
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct Stack {
     top: Piece,
@@ -45,6 +56,22 @@ pub struct Stack {
 }
 
 impl Stack {
+    /// Creates a new stack.
+    ///
+    /// # Arguments
+    ///
+    /// * `top` - The type of the top piece.
+    /// * `colors` - The colors of the pieces inside the stack, given in order from bottom to top, including the color of the top piece.
+    ///
+    /// # Panics
+    ///
+    /// The constructor panics if the `colors` iterator is empty.
+    pub fn new<I: IntoIterator<Item = Color>>(top: Piece, colors: I) -> Self {
+        let colors = Vec::from_iter(colors);
+        assert!(!colors.is_empty());
+        Self { top, colors }
+    }
+
     pub fn top(&self) -> Piece {
         self.top
     }
@@ -99,13 +126,13 @@ impl Display for Stack {
 }
 
 #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
-enum ExtendedSquare {
+pub enum ExtendedSquare {
     Stack(Stack),
     EmptySquares(usize),
 }
 
 impl ExtendedSquare {
-    pub fn iter(&self) -> Iter {
+    fn iter(&self) -> Iter {
         Iter::new(self)
     }
 }
@@ -142,16 +169,6 @@ impl Display for ExtendedSquare {
     }
 }
 
-impl<'a> IntoIterator for &'a ExtendedSquare {
-    type Item = Option<&'a Stack>;
-
-    type IntoIter = Iter<'a>;
-
-    fn into_iter(self) -> Self::IntoIter {
-        self.iter()
-    }
-}
-
 struct Iter<'a> {
     item: Option<&'a Stack>,
     count: usize,
@@ -208,6 +225,19 @@ pub struct Tps {
 }
 
 impl Tps {
+    pub unsafe fn new_unchecked(
+        mut board: Vec<Vec<ExtendedSquare>>,
+        active_player: Color,
+        full_move_number: NonZeroUsize,
+    ) -> Self {
+        canonicalize(&mut board);
+        Self {
+            board,
+            color: active_player,
+            full_move: full_move_number,
+        }
+    }
+
     pub fn board_2d(&self) -> impl Iterator<Item = impl Iterator<Item = Option<&'_ Stack>>> {
         self.board
             .iter()
@@ -252,7 +282,7 @@ fn start_position_tps(size: usize) -> String {
     )
 }
 
-fn canonicalize(board: &mut Vec<Vec<ExtendedSquare>>) {
+fn canonicalize(board: &mut [Vec<ExtendedSquare>]) {
     board.iter_mut().for_each(|row| {
         row.dedup_by(|item, acc| {
             if let ExtendedSquare::EmptySquares(acc) = acc {
@@ -282,7 +312,7 @@ impl FromStr for Tps {
             .parse()
             .map_err(|_| ParseTpsError::InvalidFullMove)?;
 
-        let board = if board == "" {
+        let board = if board.is_empty() {
             vec![]
         } else {
             let rows: Vec<_> = board.split('/').collect();
